#!/usr/bin/python
# F_waveform_envelope_spectrum_display.py. D. Gibbon, 2021-07-06

import sys, re												# import specialised modules
import numpy as np
import matplotlib.pyplot as plt
import scipy.io.wavfile as wave
from scipy.signal import medfilt, butter, lfilter
from module_spectrogram import *
from module_drawdendrogram import *

figwidth = 12
figheight = 7

boxwidth = 0.45
boxheight = 0.18
boxx = 0.55
boxy = 0.42

bincount = 20

specheatmaptype = "YlOrRd"
specgramdotsize = 60
fontsize = 10

amspecfreqmin = 0
amspecfreqmax = 5

magscount = 6

spectrumpower = 1
specdownsample = 4

# Set spectrogram moving window length for FFT analysis (default 5)
specwindowsecs = 3

# Choose number of equally spaced rows in spectrogram matrix (default 50)
# For trajectories: 250
specstrides = 100	# good idea, gives same length spectrograms; calculate duration later

#-----------------------------------------------------------------------------------------

def outputtextlines(text, filename):
	handle = open(filename,'w')
	linelist = handle.write(text)
	handle.close()
	return

def appendtextlines(text, filename):
	handle = open(filename,'a')
	linelist = handle.write(text)
	handle.close()
	return

#-----------------------------------------------------------------------------------------

if len(sys.argv) > 1:
	wavfilename = sys.argv[1]								# get input filename from command line
else:
	wavfilename = "DATA/one-to-ten-Putonghua-Lara-16k-mono.wav"
wavfilebase = re.sub("^.*/","",wavfilename)
wavfilebase = re.sub("-16k-mono","",wavfilebase[:-4])
figurefilename = "PNG/L_%s.png"%wavfilebase
csvfilename = "CSV/L_%s.csv"%wavfilebase

fs, signal = wave.read(wavfilename)							# read sampling frequency and signal
signallength = len(signal)									# define signal length in bytes
signalseconds = signallength / fs							# define signal length in seconds
signal = signal / max(abs(signal))							# normalise signal -1 ... 0 ... 1

b, a = butter(5, 5 / (0.5 * fs), btype="low")				# define Butterworth filter
envelope = lfilter(b, a, abs(signal))						# apply filter to create lf envelope
envelope = envelope / max(envelope)							# normalise envelope 0 ... 1

#-----------------------------------------------------------------------------------------

amspecmags = np.abs(np.fft.rfft(envelope))
amspecmaglen = len(amspecmags)
amspecfreqs = np.linspace(0,fs/2,amspecmaglen)

lfamspecmaglen = int(round(amspecfreqmax * amspecmaglen / (fs / 2)))
lfamspecmags = amspecmags[1:lfamspecmaglen]
lfamspecmags = lfamspecmags / np.max(lfamspecmags)
lfamspecfreqs = amspecfreqs[1:lfamspecmaglen]

amtopmagscount = magscount										# define max frequency of lf spectrum
amtopmags = sorted(lfamspecmags)[-amtopmagscount:]
amtoppos = [ list(lfamspecmags).index(m) for m in amtopmags ]
amtopfreqs = [ lfamspecfreqs[p] for p in amtoppos ]

#-----------------------------------------------------------------------------------------

#freqarray, magarray, maxfreqs, maxmags = spectrogramarrays(signal, fs)

magarray, freqarray = spectrogramarray(signal, fs,amspecfreqmin, amspecfreqmax,
							specdownsample, spectrumpower, specwindowsecs, specstrides)
print("Shape magarray:", magarray.shape)
print("Shape freqarray:", freqarray.shape)

maxmags = np.array([ max(mags[1:]) for mags in magarray ])
print(maxmags.shape)

# Loop to define spectrogram as a spectrum sequence
maxfreqs = []
for mags, freqs in zip(magarray, freqarray):
	mags = mags/max(mags)
	mags = list(mags[1:])
	maxmag = max(mags)
	maxmagpos = mags.index(maxmag)
	maxfreq = freqs[maxmagpos]
	maxfreqs += [maxfreq]
# Alternative 'pythonic' list-creating loop as a Python 'comprehension':
#maxfreqs = [ freqs[list(mags[1:]).index(max(mags[1:]))] for mags, freqs in zip(magarray, freqarray) ]

maxfreqsstring = '\t'.join([ str(maxfreq) for maxfreq in maxfreqs ])+"\n"
outputtextlines(maxfreqsstring, csvfilename)

#-----------------------------------------------------------------------------------------

fig,((plt01, plt02), (plt03, plt04), (plt05, plt06)) = plt.subplots(nrows=3, ncols=2, figsize=(figwidth, figheight))
# 9. 3D-pectrogram: heatmap

plt.suptitle("%s, fs=%d"%(wavfilename, fs), fontweight="bold")	# display a title

#-----------------------------------------------------------------------------------------
# Plot waveform and envelope here
xaxistime = np.linspace(0, signalseconds, signallength)		# define x axis in seconds
plt01.plot(xaxistime, signal, color="lightgrey")			# plot waveform in grey
plt01.plot(xaxistime, envelope, color="red")
plt01.set_xlim(0,np.ceil(signalseconds))
plt01.set_xlabel("Time")
plt01.set_title("Waveform and positive amplitude modulation envelope")
plt01.set_ylabel("Amplitude")
plt01.grid()

#-----------------------------------------------------------------------------------------

# Frequency domain
plt02.plot(lfamspecfreqs, lfamspecmags)
plt02.scatter(amtopfreqs, amtopmags, s=24, color="red")
for f,m in zip(amtopfreqs, amtopmags):
	plt02.text(f, m-0.1, "%.3fHz\n%dms"%(f,1000/f), fontsize=6)
plt02.set_title("Amplitude Modulation Low Frequency Spectrum")
plt02.set_xlabel("Frequency (Hz)")
plt02.set_ylabel("Magnitude")
plt02.set_xlim(amspecfreqmin,amspecfreqmax)

#-----------------------------------------------------------------------------------------

plotspectrogramheatmap(plt03, freqarray, magarray, signalseconds, amspecfreqmin, amspecfreqmax, specgramdotsize, specheatmaptype, fontsize)

#-----------------------------------------------------------------------------------------

# amtopmags, amtopfreqs

plt04.set_xticks([])
plt04.set_yticks([])
plt04.spines["top"].set_visible(False)
plt04.spines["bottom"].set_visible(False)
plt04.spines["left"].set_visible(False)
plt04.spines["right"].set_visible(False)

# drawdendrogram(plt04, "AM", 8, zip(amtopfreqs, amtopmags), 0.515, 0.115, fontsize)
drawdendrogram(plt04, "AM", 8, zip(amtopfreqs, amtopmags), boxwidth, boxheight, boxx, boxy, fontsize)
plt04.set_title("Rhythm formant pattern (%d values) as dendrogram"%len(amtopmags), fontsize=fontsize)

#-----------------------------------------------------------------------------------------

# plt05.plot(maxmagarray, color="green")
plt05.set_title("Vector of maximum magnitude frequencies", fontsize=fontsize)
x = np.linspace(0,signalseconds, len(maxfreqs))
plt05.scatter(x, maxfreqs, s=8)
plt05.plot(x, maxfreqs)
plt05.set_xlim(0,np.ceil(signalseconds))
plt05.set_ylim(amspecfreqmin, amspecfreqmax)
plt05.set_xlabel("Time (s)")
plt05.set_ylabel("Frequency (Hz)")
plt05.grid()

#-----------------------------------------------------------------------------------------

# plt06.set_xticks([])
plt06.set_yticks([])
plt06.spines["top"].set_visible(False)
plt06.spines["bottom"].set_visible(False)
plt06.spines["left"].set_visible(False)
plt06.spines["right"].set_visible(False)

rhythmweight = True
rhythmbars = lfamspecfreqs
weightlist = lfamspecmags

plt06.set_title("Rhythm formant pattern in %d bins"%bincount, fontsize=fontsize)

if rhythmweight:
	lfam_hist,bins_lts = np.histogram(lfamspecfreqs, weights=lfamspecmags, bins=bincount)
	plt06.hist(rhythmbars, weights=weightlist, bins=bincount, color="grey")
	weighted = "weighted"

plt06.set_xlim(amspecfreqmin,amspecfreqmax)
plt06.set_xlabel("Frequency (Hz)")

#-----------------------------------------------------------------------------------------

plt.savefig(figurefilename)
plt.tight_layout(pad=3, w_pad=0, h_pad=1)
plt.show()													# display figure
